@@ -5,6 +5,8 @@ use super::*;
 pub(crate) enum Keyword {
   Alias,
   AllowDuplicateRecipes,
+  AllowDuplicateVariables,
+  Assert,
   DotenvFilename,
   DotenvLoad,
   DotenvPath,
@@ -24,6 +26,7 @@ pub(crate) enum Keyword {
   True,
   WindowsPowershell,
   WindowsShell,
+  X,
 }
 
 impl Keyword {
@@ -41,3 +44,14 @@ impl<'a> PartialEq<&'a str> for Keyword {
     self.lexeme() == *other
   }
 }
+
+#[cfg(test)]
+mod tests {
+  use super::*;
+
+  #[test]
+  fn keyword_case() {
+    assert_eq!(Keyword::X.lexeme(), "x");
+    assert_eq!(Keyword::IgnoreComments.lexeme(), "ignore-comments");
+  }
+}
