@@ -38,16 +38,16 @@ pub(crate) struct Lexer<'src> {
 impl<'src> Lexer<'src> {
   /// Lex `src`
   pub(crate) fn lex(path: &'src Path, src: &'src str) -> CompileResult<'src, Vec<Token<'src>>> {
-    Lexer::new(path, src).tokenize()
+    Self::new(path, src).tokenize()
   }
 
   #[cfg(test)]
   pub(crate) fn test_lex(src: &'src str) -> CompileResult<'src, Vec<Token<'src>>> {
-    Lexer::new("justfile".as_ref(), src).tokenize()
+    Self::new("justfile".as_ref(), src).tokenize()
   }
 
   /// Create a new Lexer to lex `src`
-  fn new(path: &'src Path, src: &'src str) -> Lexer<'src> {
+  fn new(path: &'src Path, src: &'src str) -> Self {
     let mut chars = src.chars();
     let next = chars.next();
 
@@ -57,7 +57,7 @@ impl<'src> Lexer<'src> {
       line: 0,
     };
 
-    Lexer {
+    Self {
       indentation: vec![""],
       tokens: Vec::new(),
       token_start: start,
@@ -75,7 +75,7 @@ impl<'src> Lexer<'src> {
 
   /// Advance over the character in `self.next`, updating `self.token_end`
   /// accordingly.
-  fn advance(&mut self) -> CompileResult<'src, ()> {
+  fn advance(&mut self) -> CompileResult<'src> {
     match self.next {
       Some(c) => {
         let len_utf8 = c.len_utf8();
@@ -97,7 +97,7 @@ impl<'src> Lexer<'src> {
   }
 
   /// Advance over N characters.
-  fn skip(&mut self, n: usize) -> CompileResult<'src, ()> {
+  fn skip(&mut self, n: usize) -> CompileResult<'src> {
     for _ in 0..n {
       self.advance()?;
     }
@@ -124,7 +124,7 @@ impl<'src> Lexer<'src> {
     }
   }
 
-  fn presume(&mut self, c: char) -> CompileResult<'src, ()> {
+  fn presume(&mut self, c: char) -> CompileResult<'src> {
     if !self.next_is(c) {
       return Err(self.internal_error(format!("Lexer presumed character `{c}`")));
     }
@@ -134,7 +134,7 @@ impl<'src> Lexer<'src> {
     Ok(())
   }
 
-  fn presume_str(&mut self, s: &str) -> CompileResult<'src, ()> {
+  fn presume_str(&mut self, s: &str) -> CompileResult<'src> {
     for c in s.chars() {
       self.presume(c)?;
     }
@@ -231,11 +231,8 @@ impl<'src> Lexer<'src> {
     // The width of the error site to highlight depends on the kind of error:
     let length = match kind {
       UnterminatedString | UnterminatedBacktick => {
-        let kind = match StringKind::from_token_start(self.lexeme()) {
-          Some(kind) => kind,
-          None => {
-            return self.internal_error("Lexer::error: expected string or backtick token start")
-          }
+        let Some(kind) = StringKind::from_token_start(self.lexeme()) else {
+          return self.internal_error("Lexer::error: expected string or backtick token start");
         };
         kind.delimiter().len()
       }
@@ -282,11 +279,7 @@ impl<'src> Lexer<'src> {
 
   /// True if `c` can be a continuation character of an identifier
   fn is_identifier_continue(c: char) -> bool {
-    if Self::is_identifier_start(c) {
-      return true;
-    }
-
-    matches!(c, '0'..='9' | '-')
+    Self::is_identifier_start(c) || matches!(c, '0'..='9' | '-')
   }
 
   /// Consume the text and produce a series of tokens
@@ -328,7 +321,7 @@ impl<'src> Lexer<'src> {
   }
 
   /// Handle blank lines and indentation
-  fn lex_line_start(&mut self) -> CompileResult<'src, ()> {
+  fn lex_line_start(&mut self) -> CompileResult<'src> {
     enum Indentation<'src> {
       // Line only contains whitespace
       Blank,
@@ -478,7 +471,7 @@ impl<'src> Lexer<'src> {
   }
 
   /// Lex token beginning with `start` outside of a recipe body
-  fn lex_normal(&mut self, start: char) -> CompileResult<'src, ()> {
+  fn lex_normal(&mut self, start: char) -> CompileResult<'src> {
     match start {
       ' ' | '\t' => self.lex_whitespace(),
       '!' if self.rest().starts_with("!include") => Err(self.error(Include)),
@@ -493,10 +486,11 @@ impl<'src> Lexer<'src> {
       ',' => self.lex_single(Comma),
       '/' => self.lex_single(Slash),
       ':' => self.lex_colon(),
-      '\\' => self.lex_escape(),
       '=' => self.lex_choices('=', &[('=', EqualsEquals), ('~', EqualsTilde)], Equals),
+      '?' => self.lex_single(QuestionMark),
       '@' => self.lex_single(At),
       '[' => self.lex_delimiter(BracketL),
+      '\\' => self.lex_escape(),
       '\n' | '\r' => self.lex_eol(),
       '\u{feff}' => self.lex_single(ByteOrderMark),
       ']' => self.lex_delimiter(BracketR),
@@ -516,7 +510,7 @@ impl<'src> Lexer<'src> {
     &mut self,
     interpolation_start: Token<'src>,
     start: char,
-  ) -> CompileResult<'src, ()> {
+  ) -> CompileResult<'src> {
     if self.rest_starts_with("}}") {
       // end current interpolation
       if self.interpolation_stack.pop().is_none() {
@@ -539,7 +533,7 @@ impl<'src> Lexer<'src> {
   }
 
   /// Lex token while in recipe body
-  fn lex_body(&mut self) -> CompileResult<'src, ()> {
+  fn lex_body(&mut self) -> CompileResult<'src> {
     enum Terminator {
       Newline,
       NewlineCarriageReturn,
@@ -602,14 +596,14 @@ impl<'src> Lexer<'src> {
   }
 
   /// Lex a single-character token
-  fn lex_single(&mut self, kind: TokenKind) -> CompileResult<'src, ()> {
+  fn lex_single(&mut self, kind: TokenKind) -> CompileResult<'src> {
     self.advance()?;
     self.token(kind);
     Ok(())
   }
 
   /// Lex a double-character token
-  fn lex_double(&mut self, kind: TokenKind) -> CompileResult<'src, ()> {
+  fn lex_double(&mut self, kind: TokenKind) -> CompileResult<'src> {
     self.advance()?;
     self.advance()?;
     self.token(kind);
@@ -624,7 +618,7 @@ impl<'src> Lexer<'src> {
     first: char,
     choices: &[(char, TokenKind)],
     otherwise: TokenKind,
-  ) -> CompileResult<'src, ()> {
+  ) -> CompileResult<'src> {
     self.presume(first)?;
 
     for (second, then) in choices {
@@ -640,7 +634,7 @@ impl<'src> Lexer<'src> {
   }
 
   /// Lex an opening or closing delimiter
-  fn lex_delimiter(&mut self, kind: TokenKind) -> CompileResult<'src, ()> {
+  fn lex_delimiter(&mut self, kind: TokenKind) -> CompileResult<'src> {
     use Delimiter::*;
 
     match kind {
@@ -669,7 +663,7 @@ impl<'src> Lexer<'src> {
   }
 
   /// Pop a delimiter from the open delimiter stack and error if incorrect type
-  fn close_delimiter(&mut self, close: Delimiter) -> CompileResult<'src, ()> {
+  fn close_delimiter(&mut self, close: Delimiter) -> CompileResult<'src> {
     match self.open_delimiters.pop() {
       Some((open, _)) if open == close => Ok(()),
       Some((open, open_line)) => Err(self.error(MismatchedClosingDelimiter {
@@ -687,7 +681,7 @@ impl<'src> Lexer<'src> {
   }
 
   /// Lex a two-character digraph
-  fn lex_digraph(&mut self, left: char, right: char, token: TokenKind) -> CompileResult<'src, ()> {
+  fn lex_digraph(&mut self, left: char, right: char, token: TokenKind) -> CompileResult<'src> {
     self.presume(left)?;
 
     if self.accepted(right)? {
@@ -710,7 +704,7 @@ impl<'src> Lexer<'src> {
   }
 
   /// Lex a token starting with ':'
-  fn lex_colon(&mut self) -> CompileResult<'src, ()> {
+  fn lex_colon(&mut self) -> CompileResult<'src> {
     self.presume(':')?;
 
     if self.accepted('=')? {
@@ -724,7 +718,7 @@ impl<'src> Lexer<'src> {
   }
 
   /// Lex an token starting with '\' escape
-  fn lex_escape(&mut self) -> CompileResult<'src, ()> {
+  fn lex_escape(&mut self) -> CompileResult<'src> {
     self.presume('\\')?;
 
     // Treat newline escaped with \ as whitespace
@@ -749,7 +743,7 @@ impl<'src> Lexer<'src> {
   }
 
   /// Lex a carriage return and line feed
-  fn lex_eol(&mut self) -> CompileResult<'src, ()> {
+  fn lex_eol(&mut self) -> CompileResult<'src> {
     if self.accepted('\r')? {
       if !self.accepted('\n')? {
         return Err(self.error(UnpairedCarriageReturn));
@@ -770,7 +764,7 @@ impl<'src> Lexer<'src> {
   }
 
   /// Lex name: [a-zA-Z_][a-zA-Z0-9_]*
-  fn lex_identifier(&mut self) -> CompileResult<'src, ()> {
+  fn lex_identifier(&mut self) -> CompileResult<'src> {
     self.advance()?;
 
     while let Some(c) = self.next {
@@ -787,7 +781,7 @@ impl<'src> Lexer<'src> {
   }
 
   /// Lex comment: #[^\r\n]
-  fn lex_comment(&mut self) -> CompileResult<'src, ()> {
+  fn lex_comment(&mut self) -> CompileResult<'src> {
     self.presume('#')?;
 
     while !self.at_eol_or_eof() {
@@ -800,7 +794,7 @@ impl<'src> Lexer<'src> {
   }
 
   /// Lex whitespace: [ \t]+
-  fn lex_whitespace(&mut self) -> CompileResult<'src, ()> {
+  fn lex_whitespace(&mut self) -> CompileResult<'src> {
     while self.next_is_whitespace() {
       self.advance()?;
     }
@@ -815,10 +809,8 @@ impl<'src> Lexer<'src> {
   /// Backtick:      ``[^`]*``
   /// Cooked string: "[^"]*" # also processes escape sequences
   /// Raw string:    '[^']*'
-  fn lex_string(&mut self) -> CompileResult<'src, ()> {
-    let kind = if let Some(kind) = StringKind::from_token_start(self.rest()) {
-      kind
-    } else {
+  fn lex_string(&mut self) -> CompileResult<'src> {
+    let Some(kind) = StringKind::from_token_start(self.rest()) else {
       self.advance()?;
       return Err(self.internal_error("Lexer::lex_string: invalid string start"));
     };
@@ -975,6 +967,7 @@ mod tests {
       ParenL => "(",
       ParenR => ")",
       Plus => "+",
+      QuestionMark => "?",
       Slash => "/",
       Whitespace => " ",
 
@@ -1026,7 +1019,7 @@ mod tests {
             length,
             path: "justfile".as_ref(),
           },
-          kind: Box::new(kind),
+          kind: kind.into(),
         };
         assert_eq!(have, want);
       }
@@ -2309,10 +2302,10 @@ mod tests {
         .to_string(),
       "error: Internal error, this may indicate a bug in just: Lexer presumed character `-`
 consider filing an issue: https://github.com/casey/just/issues/new
- --> justfile:1:1
-  |
-1 | !
-  | ^"
+ ——▶ justfile:1:1
+  │
+1 │ !
+  │ ^"
     );
   }
 }
