@@ -80,6 +80,23 @@ fn skip_private_recipes() {
     .run();
 }
 
+#[test]
+fn recipes_in_submodules_can_be_chosen() {
+  Test::new()
+    .args(["--unstable", "--choose"])
+    .env("JUST_CHOOSER", "head -n10")
+    .write("bar.just", "baz:\n echo BAZ")
+    .test_round_trip(false)
+    .justfile(
+      "
+        mod bar
+      ",
+    )
+    .stderr("echo BAZ\n")
+    .stdout("BAZ\n")
+    .run();
+}
+
 #[test]
 fn skip_recipes_that_require_arguments() {
   Test::new()
@@ -152,7 +169,7 @@ fn invoke_error_function() {
       ",
     )
     .stderr_regex(
-      r"error: Chooser `/ -cu fzf --multi --preview 'just --show \{\}'` invocation failed: .*\n",
+      r#"error: Chooser `/ -cu fzf --multi --preview 'just --unstable --color always --justfile ".*justfile" --show \{\}'` invocation failed: .*\n"#,
     )
     .status(EXIT_FAILURE)
     .shell(false)
