@@ -72,10 +72,10 @@ test! {
   ",
   stderr: "
   error: Unknown setting `foo`
-   --> justfile:1:5
-    |
-  1 | set foo
-    |     ^^^
+   ——▶ justfile:1:5
+    │
+  1 │ set foo
+    │     ^^^
   ",
   status: EXIT_FAILURE,
 }
@@ -87,10 +87,10 @@ test! {
   ",
   stderr: "
   error: Unknown setting `if`
-   --> justfile:1:5
-    |
-  1 | set if := 'foo'
-    |     ^^
+   ——▶ justfile:1:5
+    │
+  1 │ set if := 'foo'
+    │     ^^
   ",
   status: EXIT_FAILURE,
 }
@@ -108,10 +108,10 @@ test! {
   justfile: "alias foo := bar\nalias foo := baz\n",
   stderr: "
     error: Alias `foo` first defined on line 1 is redefined on line 2
-     --> justfile:2:7
-      |
-    2 | alias foo := baz
-      |       ^^^
+     ——▶ justfile:2:7
+      │
+    2 │ alias foo := baz
+      │       ^^^
   ",
   status: EXIT_FAILURE,
 }
@@ -121,10 +121,10 @@ test! {
   justfile: "alias foo := bar\n",
   stderr: "
     error: Alias `foo` has an unknown target `bar`
-     --> justfile:1:7
-      |
-    1 | alias foo := bar
-      |       ^^^
+     ——▶ justfile:1:7
+      │
+    1 │ alias foo := bar
+      │       ^^^
   ",
   status: EXIT_FAILURE,
 }
@@ -134,10 +134,10 @@ test! {
   justfile: "bar:\n  echo bar\nalias foo := bar\nfoo:\n  echo foo",
   stderr: "
     error: Alias `foo` defined on line 3 is redefined as a recipe on line 4
-     --> justfile:4:1
-      |
-    4 | foo:
-      | ^^^
+     ——▶ justfile:4:1
+      │
+    4 │ foo:
+      │ ^^^
   ",
   status: EXIT_FAILURE,
 }
@@ -187,41 +187,6 @@ c: b
   stderr:   "echo a\necho b\necho c\necho d\n",
 }
 
-test! {
-  name:     summary,
-  justfile: "b: a
-a:
-d: c
-c: b
-_z: _y
-_y:
-",
-  args:     ("--summary"),
-  stdout:   "a b c d\n",
-}
-
-test! {
-  name:     summary_sorted,
-  justfile: "
-b:
-c:
-a:
-",
-  args:     ("--summary"),
-  stdout:   "a b c\n",
-}
-
-test! {
-  name:     summary_unsorted,
-  justfile: "
-b:
-c:
-a:
-",
-  args:     ("--summary", "--unsorted"),
-  stdout:   "b c a\n",
-}
-
 test! {
   name:     select,
   justfile: "b:
@@ -269,10 +234,10 @@ test! {
   justfile: "bar:\nhello:\nfoo: bar baaaaaaaz hello",
   stderr:   "
     error: Recipe `foo` has unknown dependency `baaaaaaaz`
-     --> justfile:3:10
-      |
-    3 | foo: bar baaaaaaaz hello
-      |          ^^^^^^^^^
+     ——▶ justfile:3:10
+      │
+    3 │ foo: bar baaaaaaaz hello
+      │          ^^^^^^^^^
   ",
   status:   EXIT_FAILURE,
 }
@@ -296,10 +261,10 @@ test! {
   justfile: "b := a\na := `exit 100`\nbar:\n echo '{{`exit 200`}}'",
   stderr:   "
     error: Backtick failed with exit code 100
-     --> justfile:2:6
-      |
-    2 | a := `exit 100`
-      |      ^^^^^^^^^^
+     ——▶ justfile:2:6
+      │
+    2 │ a := `exit 100`
+      │      ^^^^^^^^^^
   ",
   status:   100,
 }
@@ -309,10 +274,10 @@ test! {
   justfile: "b := a\na := `echo hello`\nbar:\n echo '{{`exit 200`}}'",
   stderr:   "
     error: Backtick failed with exit code 200
-     --> justfile:4:10
-      |
-    4 |  echo '{{`exit 200`}}'
-      |          ^^^^^^^^^^
+     ——▶ justfile:4:10
+      │
+    4 │  echo '{{`exit 200`}}'
+      │          ^^^^^^^^^^
   ",
   status:   200,
 }
@@ -322,10 +287,10 @@ test! {
   justfile: "f:\n 無{{`exit 200`}}",
   stderr:   "
     error: Backtick failed with exit code 200
-     --> justfile:2:7
-      |
-    2 |  無{{`exit 200`}}
-      |      ^^^^^^^^^^
+     ——▶ justfile:2:7
+      │
+    2 │  無{{`exit 200`}}
+      │      ^^^^^^^^^^
   ",
   status:   200,
 }
@@ -337,10 +302,10 @@ test! {
     \techo {{`exit 200`}}
   ",
   stderr:   "    error: Backtick failed with exit code 200
-     --> justfile:2:9
-      |
-    2 |     echo {{`exit 200`}}
-      |            ^^^^^^^^^^
+     ——▶ justfile:2:9
+      │
+    2 │     echo {{`exit 200`}}
+      │            ^^^^^^^^^^
   ",
   status:   200,
 }
@@ -352,10 +317,10 @@ test! {
     \techo {{\t`exit 200`}}
   ",
   stderr:   "error: Backtick failed with exit code 200
- --> justfile:2:10
-  |
-2 |     echo {{    `exit 200`}}
-  |                ^^^^^^^^^^
+ ——▶ justfile:2:10
+  │
+2 │     echo {{    `exit 200`}}
+  │                ^^^^^^^^^^
 ",
   status:   200,
 }
@@ -368,10 +333,10 @@ test! {
   ",
   stderr:   "
     error: Backtick failed with exit code 200
-     --> justfile:2:10
-      |
-    2 |     echo {{    `exit        200`}}
-      |                ^^^^^^^^^^^^^^^^^
+     ——▶ justfile:2:10
+      │
+    2 │     echo {{    `exit        200`}}
+      │                ^^^^^^^^^^^^^^^^^
   ",
   status:   200,
 }
@@ -384,10 +349,10 @@ test! {
   ",
   stderr: "
     error: Backtick failed with exit code 200
-     --> justfile:2:13
-      |
-    2 |     echo 😬{{`exit 200`}}
-      |              ^^^^^^^^^^
+     ——▶ justfile:2:13
+      │
+    2 │     echo 😬{{`exit 200`}}
+      │              ^^^^^^^^^^
   ",
   status:   200,
 }
@@ -400,10 +365,10 @@ test! {
   ",
   stderr: "
     error: Backtick failed with exit code 200
-     --> justfile:2:24
-      |
-    2 |     echo             😬鎌鼬{{        `exit 200 #             abc`}}            😬鎌鼬
-      |                                      ^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+     ——▶ justfile:2:24
+      │
+    2 │     echo             😬鎌鼬{{        `exit 200 #             abc`}}            😬鎌鼬
+      │                                      ^^^^^^^^^^^^^^^^^^^^^^^^^^^^
   ",
   status:   200,
 }
@@ -424,10 +389,10 @@ test! {
   ",
   stderr:   "
     error: Backtick failed with exit code 200
-      --> justfile:10:10
-       |
-    10 |  echo '{{`exit 200`}}'
-       |          ^^^^^^^^^^
+      ——▶ justfile:10:10
+       │
+    10 │  echo '{{`exit 200`}}'
+       │          ^^^^^^^^^^
   ",
   status:   200,
 }
@@ -441,10 +406,10 @@ test! {
   stdout:   "",
   stderr:   "
     error: Backtick failed with exit code 123
-     --> justfile:4:9
-      |
-    4 |  echo {{`exit 123`}}
-      |         ^^^^^^^^^^
+     ——▶ justfile:4:9
+      │
+    4 │  echo {{`exit 123`}}
+      │         ^^^^^^^^^^
   ",
   status:   123,
 }
@@ -458,10 +423,10 @@ test! {
   stderr:   "
     echo hello
     error: Backtick failed with exit code 123
-     --> justfile:3:9
-      |
-    3 |  echo {{`exit 123`}}
-      |         ^^^^^^^^^^
+     ——▶ justfile:3:9
+      │
+    3 │  echo {{`exit 123`}}
+      │         ^^^^^^^^^^
   ",
   status:   123,
 }
@@ -475,10 +440,10 @@ a := `exit 222`",
   stdout:   "",
   stderr:   "
     error: Backtick failed with exit code 222
-     --> justfile:4:6
-      |
-    4 | a := `exit 222`
-      |      ^^^^^^^^^^
+     ——▶ justfile:4:6
+      │
+    4 │ a := `exit 222`
+      │      ^^^^^^^^^^
   ",
   status:   222,
 }
@@ -587,14 +552,14 @@ test! {
 
 
 
-???
+^^^
 "#,
   stdout:   "",
   stderr:   "error: Unknown start of token:
-  --> justfile:10:1
-   |
-10 | ???
-   | ^
+  ——▶ justfile:10:1
+   │
+10 │ ^^^
+   │ ^
 ",
   status:   EXIT_FAILURE,
 }
@@ -696,7 +661,7 @@ test! {
   justfile: "b := a\na := `exit 100`\nbar:\n echo '{{`exit 200`}}'",
   args:     ("--color", "always"),
   stdout:   "",
-  stderr:   "\u{1b}[1;31merror\u{1b}[0m: \u{1b}[1mBacktick failed with exit code 100\u{1b}[0m\n \u{1b}[1;34m-->\u{1b}[0m justfile:2:6\n  \u{1b}[1;34m|\u{1b}[0m\n\u{1b}[1;34m2 |\u{1b}[0m a := `exit 100`\n  \u{1b}[1;34m|\u{1b}[0m      \u{1b}[1;31m^^^^^^^^^^\u{1b}[0m\n",
+  stderr:   "\u{1b}[1;31merror\u{1b}[0m: \u{1b}[1mBacktick failed with exit code 100\u{1b}[0m\n \u{1b}[1;34m——▶\u{1b}[0m justfile:2:6\n  \u{1b}[1;34m│\u{1b}[0m\n\u{1b}[1;34m2 │\u{1b}[0m a := `exit 100`\n  \u{1b}[1;34m│\u{1b}[0m      \u{1b}[1;31m^^^^^^^^^^\u{1b}[0m\n",
   status:   100,
 }
 
@@ -706,10 +671,10 @@ test! {
   args:     ("--color", "never"),
   stdout:   "",
   stderr:   "error: Backtick failed with exit code 100
- --> justfile:2:6
-  |
-2 | a := `exit 100`
-  |      ^^^^^^^^^^
+ ——▶ justfile:2:6
+  │
+2 │ a := `exit 100`
+  │      ^^^^^^^^^^
 ",
   status:   100,
 }
@@ -720,10 +685,10 @@ test! {
   args:     ("--color", "auto"),
   stdout:   "",
   stderr:   "error: Backtick failed with exit code 100
- --> justfile:2:6
-  |
-2 | a := `exit 100`
-  |      ^^^^^^^^^^
+ ——▶ justfile:2:6
+  │
+2 │ a := `exit 100`
+  │      ^^^^^^^^^^
 ",
   status:   100,
 }
@@ -759,10 +724,10 @@ test! {
   stdout:   "",
   stderr:   "error: Found a mix of tabs and spaces in leading whitespace: `␉␠`
 Leading whitespace may consist of tabs or spaces, but not both
- --> justfile:2:1
-  |
-2 |      echo hello
-  | ^^^^^
+ ——▶ justfile:2:1
+  │
+2 │      echo hello
+  │ ^^^^^
 ",
   status:   EXIT_FAILURE,
 }
@@ -772,10 +737,10 @@ test! {
   justfile: "bar:\n\t\techo hello\n\t\t\techo goodbye",
   stdout:   "",
   stderr:   "error: Recipe line has extra leading whitespace
- --> justfile:3:3
-  |
-3 |             echo goodbye
-  |         ^^^^^^^^^^^^^^^^
+ ——▶ justfile:3:3
+  │
+3 │             echo goodbye
+  │         ^^^^^^^^^^^^^^^^
 ",
   status:   EXIT_FAILURE,
 }
@@ -786,10 +751,10 @@ test! {
   stdout:   "",
   stderr:   "error: Recipe line has inconsistent leading whitespace. \
             Recipe started with `␉␉` but found line with `␉␠`
- --> justfile:3:1
-  |
-3 |      echo goodbye
-  | ^^^^^
+ ——▶ justfile:3:1
+  │
+3 │      echo goodbye
+  │ ^^^^^
 ",
   status:   EXIT_FAILURE,
 }
@@ -799,10 +764,10 @@ test! {
   justfile: "bar:\nhello baz arg='foo' bar:",
   stdout:   "",
   stderr:   "error: Non-default parameter `bar` follows default parameter
- --> justfile:2:21
-  |
-2 | hello baz arg='foo' bar:
-  |                     ^^^
+ ——▶ justfile:2:21
+  │
+2 │ hello baz arg='foo' bar:
+  │                     ^^^
 ",
   status:   EXIT_FAILURE,
 }
@@ -812,10 +777,10 @@ test! {
   justfile: "bar:\nhello baz +arg bar:",
   stdout:   "",
   stderr:   "error: Parameter `bar` follows variadic parameter
- --> justfile:2:16
-  |
-2 | hello baz +arg bar:
-  |                ^^^
+ ——▶ justfile:2:16
+  │
+2 │ hello baz +arg bar:
+  │                ^^^
 ",
   status:   EXIT_FAILURE,
 }
@@ -825,10 +790,10 @@ test! {
   justfile: "bar:\nhello baz *arg bar:",
   stdout:   "",
   stderr:   "error: Parameter `bar` follows variadic parameter
- --> justfile:2:16
-  |
-2 | hello baz *arg bar:
-  |                ^^^
+ ——▶ justfile:2:16
+  │
+2 │ hello baz *arg bar:
+  │                ^^^
 ",
   status:   EXIT_FAILURE,
 }
@@ -937,7 +902,7 @@ x a b='B	' c='C':
   echo {{a}} {{b}} {{c}}
 
 # something else
-this-recipe-is-very-very-very-important Z="\t z":
+this-recipe-is-very-very-very-very-very-very-very-very-important Z="\t z":
 
 # this recipe will not appear
 _private-recipe:
@@ -946,7 +911,7 @@ _private-recipe:
   stdout:   r#"
     Available recipes:
         hello a b='B	' c='C' # this does a thing
-        this-recipe-is-very-very-very-important Z="\t z" # something else
+        this-recipe-is-very-very-very-very-very-very-very-very-important Z="\t z" # something else
         x a b='B	' c='C'     # this does another thing
   "#,
 }
@@ -1198,10 +1163,10 @@ bar:"#,
   args:     ("bar"),
   stdout:   "",
   stderr:   r#"error: Call to unknown function `foo`
- --> justfile:1:8
-  |
-1 | foo := foo() + "hello"
-  |        ^^^
+ ——▶ justfile:1:8
+  │
+1 │ foo := foo() + "hello"
+  │        ^^^
 "#,
   status:   EXIT_FAILURE,
 }
@@ -1215,10 +1180,10 @@ test! {
   args:     ("b"),
   stdout:   "",
   stderr:   "error: Dependency `a` got 0 arguments but takes 1 argument
- --> justfile:2:4
-  |
-2 | b: a
-  |    ^
+ ——▶ justfile:2:4
+  │
+2 │ b: a
+  │    ^
 ",
   status:   EXIT_FAILURE,
 }
@@ -1232,10 +1197,10 @@ test! {
   args:     ("b"),
   stdout:   "",
   stderr:   "error: Dependency `a` got 0 arguments but takes at least 1 argument
- --> justfile:2:4
-  |
-2 | b: a
-  |    ^
+ ——▶ justfile:2:4
+  │
+2 │ b: a
+  │    ^
 ",
   status:   EXIT_FAILURE,
 }
@@ -1249,10 +1214,10 @@ test! {
   args:     ("b"),
   stdout:   "",
   stderr:   "error: Dependency `a` got 3 arguments but takes at most 2 arguments
- --> justfile:2:5
-  |
-2 | b: (a '0' '1' '2')
-  |     ^
+ ——▶ justfile:2:5
+  │
+2 │ b: (a '0' '1' '2')
+  │     ^
 ",
   status:   EXIT_FAILURE,
 }
@@ -1263,10 +1228,10 @@ test! {
   args:     ("a"),
   stdout:   "",
   stderr:   "error: Recipe `a` has duplicate parameter `foo`
- --> justfile:1:7
-  |
-1 | a foo foo:
-  |       ^^^
+ ——▶ justfile:1:7
+  │
+1 │ a foo foo:
+  │       ^^^
 ",
   status:   EXIT_FAILURE,
 }
@@ -1277,10 +1242,10 @@ test! {
   args:     ("b"),
   stdout:   "",
   stderr:   "error: Recipe `b` first defined on line 1 is redefined on line 2
- --> justfile:2:1
-  |
-2 | b:
-  | ^
+ ——▶ justfile:2:1
+  │
+2 │ b:
+  │ ^
 ",
   status:   EXIT_FAILURE,
 }
@@ -1291,10 +1256,10 @@ test! {
   args:     ("foo"),
   stdout:   "",
   stderr:   "error: Variable `a` has multiple definitions
- --> justfile:2:1
-  |
-2 | a := 'hello'
-  | ^
+ ——▶ justfile:2:1
+  │
+2 │ a := 'hello'
+  │ ^
 ",
   status:   EXIT_FAILURE,
 }
@@ -1306,10 +1271,10 @@ test! {
   stdout:   "",
   stderr:   "error: Expected '&&', comment, end of file, end of line, \
     identifier, or '(', but found string
- --> justfile:1:6
-  |
-1 | foo: 'bar'
-  |      ^^^^^
+ ——▶ justfile:1:6
+  │
+1 │ foo: 'bar'
+  │      ^^^^^
 ",
   status:   EXIT_FAILURE,
 }
@@ -1320,10 +1285,10 @@ test! {
   args:     ("foo"),
   stdout:   "",
   stderr:   "error: Expected '*', ':', '$', identifier, or '+', but found string
- --> justfile:1:5
-  |
-1 | foo 'bar'
-  |     ^^^^^
+ ——▶ justfile:1:5
+  │
+1 │ foo 'bar'
+  │     ^^^^^
 ",
   status:   EXIT_FAILURE,
 }
@@ -1334,10 +1299,10 @@ test! {
   args:     ("a"),
   stdout:   "",
   stderr:   "error: Recipe `a` depends on itself
- --> justfile:1:4
-  |
-1 | a: a
-  |    ^
+ ——▶ justfile:1:4
+  │
+1 │ a: a
+  │    ^
 ",
   status:   EXIT_FAILURE,
 }
@@ -1348,10 +1313,10 @@ test! {
   args:     ("a"),
   stdout:   "",
   stderr:   "error: Recipe `d` has circular dependency `a -> b -> c -> d -> a`
- --> justfile:4:4
-  |
-4 | d: a
-  |    ^
+ ——▶ justfile:4:4
+  │
+4 │ d: a
+  │    ^
 ",
   status:   EXIT_FAILURE,
 }
@@ -1362,10 +1327,10 @@ test! {
   args:     ("a"),
   stdout:   "",
   stderr:   "error: Variable `z` is defined in terms of itself
- --> justfile:1:1
-  |
-1 | z := z
-  | ^
+ ——▶ justfile:1:1
+  │
+1 │ z := z
+  │ ^
 ",
   status:   EXIT_FAILURE,
 }
@@ -1376,10 +1341,10 @@ test! {
   args:     ("a"),
   stdout:   "",
   stderr:   "error: Variable `x` depends on its own value: `x -> y -> z -> x`
- --> justfile:1:1
-  |
-1 | x := y
-  | ^
+ ——▶ justfile:1:1
+  │
+1 │ x := y
+  │ ^
 ",
   status:   EXIT_FAILURE,
 }
@@ -1396,10 +1361,10 @@ test! {
   args:     ("a"),
   stdout:   "",
   stderr:   "error: Variable `x` depends on its own value: `x -> y -> x`
- --> justfile:2:1
-  |
-2 | x := y
-  | ^
+ ——▶ justfile:2:1
+  │
+2 │ x := y
+  │ ^
 ",
   status:   EXIT_FAILURE,
 }
@@ -1501,10 +1466,10 @@ foo *a +b:
 ",
   stdout:   "",
   stderr:   "error: Expected \':\' or \'=\', but found \'+\'
- --> justfile:1:8
-  |
-1 | foo *a +b:
-  |        ^
+ ——▶ justfile:1:8
+  │
+1 │ foo *a +b:
+  │        ^
 ",
   status:   EXIT_FAILURE,
 }
@@ -1517,10 +1482,10 @@ foo +a *b:
 ",
   stdout:   "",
   stderr:   "error: Expected \':\' or \'=\', but found \'*\'
- --> justfile:1:8
-  |
-1 | foo +a *b:
-  |        ^
+ ——▶ justfile:1:8
+  │
+1 │ foo +a *b:
+  │        ^
 ",
   status:   EXIT_FAILURE,
 }
@@ -1551,10 +1516,10 @@ a: x y
 ",
   stdout:   "",
   stderr:   "error: Recipe `a` has unknown dependency `y`
- --> justfile:3:6
-  |
-3 | a: x y
-  |      ^
+ ——▶ justfile:3:6
+  │
+3 │ a: x y
+  │      ^
 ",
   status:   EXIT_FAILURE,
 }
@@ -1709,10 +1674,10 @@ X := "\'"
 "#,
    stdout:   "",
    stderr:   r#"error: `\'` is not a valid escape sequence
- --> justfile:1:6
-  |
-1 | X := "\'"
-  |      ^^^^
+ ——▶ justfile:1:6
+  │
+1 │ X := "\'"
+  │      ^^^^
 "#,
    status:   EXIT_FAILURE,
 }
@@ -1724,10 +1689,10 @@ test! {
    ",
    stdout:   "",
    stderr:   r#"error: Variable `bar` not defined
- --> justfile:1:7
-  |
-1 | foo x=bar:
-  |       ^^^
+ ——▶ justfile:1:7
+  │
+1 │ foo x=bar:
+  │       ^^^
 "#,
    status:   EXIT_FAILURE,
 }
@@ -1739,10 +1704,10 @@ foo x=bar():
 ",
    stdout:   "",
    stderr:   r#"error: Call to unknown function `bar`
- --> justfile:1:7
-  |
-1 | foo x=bar():
-  |       ^^^
+ ——▶ justfile:1:7
+  │
+1 │ foo x=bar():
+  │       ^^^
 "#,
    status:   EXIT_FAILURE,
 }
@@ -1796,10 +1761,10 @@ test! {
   ",
   stderr:   r#"
     error: Unterminated interpolation
-     --> justfile:2:8
-      |
-    2 |   echo {{
-      |        ^^
+     ——▶ justfile:2:8
+      │
+    2 │   echo {{
+      │        ^^
   "#,
   status:   EXIT_FAILURE,
 }
@@ -1812,10 +1777,10 @@ test! {
   ",
   stderr:   r#"
     error: Unterminated interpolation
-     --> justfile:2:8
-      |
-    2 |   echo {{
-      |        ^^
+     ——▶ justfile:2:8
+      │
+    2 │   echo {{
+      │        ^^
   "#,
   status:   EXIT_FAILURE,
 }
@@ -1827,10 +1792,10 @@ assembly_source_files = %(wildcard src/arch/$(arch)/*.s)
 ",
   stderr:   r#"
     error: Unknown start of token:
-     --> justfile:1:25
-      |
-    1 | assembly_source_files = %(wildcard src/arch/$(arch)/*.s)
-      |                         ^
+     ——▶ justfile:1:25
+      │
+    1 │ assembly_source_files = %(wildcard src/arch/$(arch)/*.s)
+      │                         ^
   "#,
    status:   EXIT_FAILURE,
 }
@@ -1926,10 +1891,10 @@ test! {
   ",
   stderr: "
     error: Expected '*', ':', '$', identifier, or '+', but found '='
-     --> justfile:1:5
-      |
-    1 | foo = 'bar'
-      |     ^
+     ——▶ justfile:1:5
+      │
+    1 │ foo = 'bar'
+      │     ^
     ",
   status: EXIT_FAILURE,
 }
@@ -2124,10 +2089,10 @@ test! {
   stdout: "",
   stderr: "
     error: Variable `a` not defined
-     --> justfile:3:9
-      |
-    3 | bar a b=a:
-      |         ^
+     ——▶ justfile:3:9
+      │
+    3 │ bar a b=a:
+      │         ^
   ",
   status: EXIT_FAILURE,
   shell: false,
